@@ -8,4 +8,8 @@ def get_json_from_file(folder_name, json_filename):
 
 def get_data_from_file(folder_name, json_filename):
     with open(Path(os.path.abspath(folder_name), json_filename)) as f:
-        return f.read()
\ No newline at end of file
+        return f.read()
+
+def write_data_to_file(folder_name, json_filename, data):
+    with open(Path(os.path.abspath(folder_name), json_filename), 'w') as f:
+        json.dump(f, data)
\ No newline at end of file
